@@ -19,6 +19,9 @@
 #include <stdio.h>
 #include <string.h>
 #include <unistd.h>
+#ifdef _WIN32
+#include <io.h>
+#endif
 
 #include <hashinc>
 
