@@ -141,11 +141,17 @@ __fdnlist(int fd, struct nlist *list)
 	    fstat(fd, &st) < 0)
 		return (-1);
 
+	if (ehdr.e_shnum == 0 ||
+	    ehdr.e_shentsize != sizeof(Elf_Shdr)) {
+		errno = ERANGE;
+		return (-1);
+	}
+
 	/* calculate section header table size */
 	shdr_size = ehdr.e_shentsize * ehdr.e_shnum;
 
 	/* Make sure it's not too big to mmap */
-	if (shdr_size > SIZE_T_MAX) {
+	if (shdr_size > SIZE_T_MAX || shdr_size > st.st_size) {
 		errno = EFBIG;
 		return (-1);
 	}
@@ -155,7 +161,7 @@ __fdnlist(int fd, struct nlist *list)
 		return (-1);
 
 	/* Load section header table. */
-	if (pread(fd, shdr, (size_t)shdr_size, (off_t)ehdr.e_shoff) < 0)
+	if (pread(fd, shdr, (size_t)shdr_size, (off_t)ehdr.e_shoff) != (ssize_t)shdr_size)
 		goto done;
 
 	/*
@@ -166,6 +172,9 @@ __fdnlist(int fd, struct nlist *list)
 	 */
 	for (i = 0; i < ehdr.e_shnum; i++) {
 		if (shdr[i].sh_type == SHT_SYMTAB) {
+			if (shdr[i].sh_link >= ehdr.e_shnum)
+				goto done;
+
 			symoff = shdr[i].sh_offset;
 			symsize = shdr[i].sh_size;
 			symstroff = shdr[shdr[i].sh_link].sh_offset;
@@ -175,7 +184,7 @@ __fdnlist(int fd, struct nlist *list)
 	}
 
 	/* Check for files too large to mmap. */
-	if (symstrsize > SIZE_T_MAX) {
+	if (symstrsize > SIZE_T_MAX || symstrsize > st.st_size) {
 		errno = EFBIG;
 		goto done;
 	}
@@ -189,7 +198,7 @@ __fdnlist(int fd, struct nlist *list)
 	if (strtab == NULL)
 		goto done;
 
-	if (pread(fd, strtab, (size_t)symstrsize, (off_t)symstroff) < 0)
+	if (pread(fd, strtab, (size_t)symstrsize, (off_t)symstroff) != (ssize_t)symstrsize)
 		goto done;
 
 	/*
@@ -227,16 +236,18 @@ __fdnlist(int fd, struct nlist *list)
 		symsize -= cc;
 		for (s = sbuf; cc > 0 && nent > 0; ++s, cc -= sizeof(*s)) {
 			char *name;
+			Elf_Word size;
 			struct nlist *p;
 
 			name = strtab + s->st_name;
 			if (name[0] == '\0')
 				continue;
+			size = symstrsize - s->st_name;
 
 			for (p = list; !ISLAST(p); p++) {
 				if ((p->n_un.n_name[0] == '_' &&
-				    strcmp(name, p->n_un.n_name+1) == 0)
-				    || strcmp(name, p->n_un.n_name) == 0) {
+				     strncmp(name, p->n_un.n_name+1, size) == 0) ||
+				    strncmp(name, p->n_un.n_name, size) == 0) {
 					elf_sym_to_nlist(p, s, shdr,
 					    ehdr.e_shnum);
 					if (--nent <= 0)
@@ -258,6 +269,10 @@ nlist(const char *name, struct nlist *list)
 {
 	int fd, n;
 
+	if (list == NULL) {
+		errno = EINVAL;
+		return (-1);
+	}
 	fd = open(name, O_RDONLY, 0);
 	if (fd < 0)
 		return (-1);
