@@ -12,14 +12,14 @@ incdir="${top_srcdir}/include/bsd"
 CPPFLAGS="$CPPFLAGS -DLIBBSD_OVERLAY"
 
 for inc in $(cd $incdir; find -name '*.h' | sort | cut -c3-); do
-  cat >headers-gen.c <<SOURCE
+  cat >headers-overlay-gen.c <<SOURCE
 #include <$inc>
 int main() { return 0; }
 SOURCE
 
   echo "testing header $inc"
-  run $CC -isystem "$incdir" $CPPFLAGS headers-gen.c -o /dev/null
+  run $CC -isystem "$incdir" $CPPFLAGS headers-overlay-gen.c -o /dev/null
   echo
 
-  rm -f headers-gen*
+  rm -f headers-overlay-gen*
 done
