@@ -11,14 +11,14 @@ run()
 incdir="${top_srcdir}/include"
 
 for inc in $(cd $incdir; find -name '*.h' | sort | cut -c3-); do
-  cat >headers-gen.c <<SOURCE
+  cat >headers-system-gen.c <<SOURCE
 #include <$inc>
 int main() { return 0; }
 SOURCE
 
   echo "testing header $inc"
-  run $CC -isystem "$incdir" $CPPFLAGS headers-gen.c -o /dev/null
+  run $CC -isystem "$incdir" $CPPFLAGS headers-system-gen.c -o /dev/null
   echo
 
-  rm -f headers-gen.*
+  rm -f headers-system-gen.*
 done
